
;Input:
;	A = signal byte:
;		$00 = 'or' lines w/background
;		$01 = 'and' lines
;		$02 = 'xor' lines
;	(B,C) = (X,Y) of box
;	D = height of box
;	E = width of box
;Output:
;	box is drawn according to dimensions
;	A,C,HL destroyed
;	B = 0
;	DE = $0010
DrawBox:
 push de
 push af
 push bc
 call &DrawHorzLine
 pop bc
 pop af
 push bc
 call &DrawVertLine
 pop bc
 pop de
 push bc
 push de
 ld a,c
 add a,d
 ld c,a
 dec c
 call &DHL_SkipOPs
 pop de
 pop bc
 ld a,b
 add a,e
 ld b,a
 dec b
 jr DVL_SkipOPs

;Input:
;	A = signal byte:
;		$00 = 'or' lines w/background
;		$01 = 'and' lines
;		$02 = 'xor' lines
;	(B,C) = (X,Y)
;	D = length of line
;Ouput:
;	if CF, line out of screen
;	line is clipped as needed
;	A destroyed
;	B = 0
;	C = bit offset of line
;	DE = $0010
;	HL -> byte in vid mem to put next pixel
DrawVertLine:
 call &LoadOPs
 ld (&DVL_OP1),a
 ld a,(hl)
 ld (&DVL_OP2),a
DVL_SkipOPs:
 ld a,b
 cp 128
 ccf
 ret c
 ld a,c
 cp 64
 jr c,DVL_YNotNeg
 add a,d
 ret z
 ccf
 ret c
 ld d,a
 xor a
 ld c,a
DVL_YNotNeg:
 add a,d
 sub 64
 jr c,DVL_YNotOff
 sub d
 neg
 ld d,a
DVL_YNotOff:
 ld a,63
 sub c
 ld c,a
 call FIND_PIXEL
 dec h\ dec h\ dec h\ dec h
 ld b,d
DVL_OP1:
 nop
 ld c,a
 ld de,$0010
DVL_NewPix:
 ld a,c
DVL_OP2:
 or (hl)
 ld (hl),a
 add hl,de
 djnz DVL_NewPix
 ret


;Input:
;	A = signal byte:
;		$00 = 'or' lines w/background
;		$01 = 'and' lines
;		$02 = 'xor' lines
;	(B,C) = (X,Y)
;	E = length of line
;Ouput:
;	if CF, line out of screen
;	line is clipped as needed
;	A destroyed
;	B = 0
;	C = bit offset of next pixel
;	E = length of drawn line
;	D preserved
;	HL -> byte in vid mem to put next pixel
DrawHorzLine:
 call &LoadOPs
 ld (&DHL_OP1),a
 ld a,(hl)\ inc hl
 ld (&DHL_OP2),a
 ld a,(hl)
 ld (&DHL_OP3),a
DHL_SkipOPs:
 ld a,63
 sub c
 ret c
 ld c,a
 ld a,b
 cp 128
 jr c,DHL_XNotNeg
 add a,e
 ret z
 ccf
 ret c
 ld e,a
 xor a
 ld b,a
DHL_XNotNeg:
 add a,e
 sub 128
 jr c,DHL_XNotOff
 sub e
 neg
 ld e,a
DHL_XNotOff:
 call FIND_PIXEL
 dec h\ dec h\ dec h\ dec h
 ld b,e
DHL_OP1:
 nop
 ld c,a
DHL_NewPix:
 ld a,c
DHL_OP2:
 or (hl)
 ld (hl),a
 rrc c
DHL_OP3:
 jr nc,DHL_NoWrap
 inc l
DHL_NoWrap:
 djnz DHL_NewPix
 ret


;Input:
;	A = signal byte:
;		$00 = 'or' lines w/background
;		$01 = 'and' lines
;		$02 = 'xor' lines
;	(B,C) = (X,Y) of box
;	D = height of box
;	E = width of box
;Output:
;	box is filled according to given parameters
;	AF,BC,DE,HL destroyed
FillBox:
 call &LoadOPs
 ld (&DVL_OP1),a
 ld (&DHL_OP1),a
 ld a,(hl)\ inc hl
 ld (&DVL_OP2),a
 ld (&DHL_OP2),a
 ld a,(hl)
 ld (&DHL_OP3),a

 ld a,d
 cp e
 jr c,FB_HorzLoop
FB_VertLoop:
 push bc
 push de
 call &DVL_SkipOPs
 pop de
 pop bc
 inc b
 dec e
 jr nz,FB_VertLoop
 ret
FB_HorzLoop:
 push bc
 call &DHL_SkipOPs
 pop bc
 inc c
 dec d
 jr nz,FB_HorzLoop
 ret



LoadOPs:
 and $03
 ld l,a
 add a,a
 add a,l
 ld hl,&LineOPs
 add a,l
 ld l,a
 jr nc,$+3
 inc h
 ld a,(hl)\ inc hl
 ret

LineOPs:
 nop\ or (hl)\.db $30
 cpl\ and (hl)\.db $38
 nop\ xor (hl)\.db $30
