
;Input:
;	($800C) = row to display entered string
;	($800D) = column to display entered string
;	(INP_STRADDR) = address to write string to
;	(INP_MAXLENGTH) = maximum length
InputString:
 ld a,($800D)
 ld (INP_STARTCOL),a	;save the starting column of the cursor
 ld c,a
 ld a,(INP_MAXLENGTH)	;A = max length of string
 ld b,a
 add a,c		;add starting column to max length
 sub 21
 jr c,IS_NoWrap	;if (INP_STARTCOL) + (INP_MAXLENGTH) < 21, jump
 sub b
 cpl			;negate and sub 1
 ld (INP_MAXLENGTH),a	;save as new max
IS_NoWrap:

 set 2,(iy+$0C)	;cursor flash enabled
 res 5,(iy+$0D)	;graph cursor disabled
 res 6,(iy+$0D)	;MODE-type cursor disabled

IS_Loop:
 call &GetKey3

 inc b
 jr nz,IS_NoChar
 inc c
 dec c
 jr z,IS_NoChar
 cp $06		;$06 = keycode2 for ENTER
 jr z,IS_Done
 ld e,c		;save char to put into E
 bit 4,(iy+$05)	;check if insert cursor
 jr z,IS_PutChar
IS_InsertChar:
 call &CheckLength
 jr nc,IS_Loop	;if the inputted length is >= the max, jump
 push hl
 call &GetStrPoint	;HL -> spot in string where cursor is
 ld a,e		;A = E = char to put
 pop de		;DE -> byte after 0 in string
 ex de,hl
 sbc hl,de
 ld c,l		;C = difference between pointers (bytes to shift)
 add hl,de
 ld d,h
 ld e,l
 dec hl
 lddr			;shift string right
 ex de,hl
 ld (hl),a		;load char into string
 ld a,($800D)	;save column cursor is at
 call D_ZT_STR	;display the rest of the chars
 ld ($800D),a	;restore column
 jr IS_Loop
IS_PutChar:
 call &GetStrPoint
 jr nz,IS_NoExtend	;jump if HL !-> nul terminator
 call &CheckLength	;check if can add one more char
 jr nc,IS_Loop	;jump if can't
 ld (hl),b		;load nul terminator
 dec hl		;HL -> where to put new char
IS_NoExtend:
 ld (hl),e		;load char
 ld a,e
 call TR_CHARPUT	;put char
 ld hl,$800D
 inc (hl)
 jr IS_Loop

IS_Done:
 res 2,(iy+$0C)	;turn cursor off
 ret

IS_NoChar:
 dec a
 jr z,GoRight	;$01
 dec a
 jr z,GoLeft	;$02
 dec a
 jr z,GoUp		;$03
 dec a
 jr z,GoDown	;$04
 sub 7 - 4
 scf
 jr z,IS_Done	;$07
 dec a
 jr z,GoClear	;$08
 dec a
 jr z,GoDelete	;$09
 dec a
 jr nz,IS_LoopLink	;$0A
GoInsert:
 ld hl,$834B
 ld a,(hl)
 xor $10
 ld (hl),a
 jr IS_LoopLink

GoRight:
 call &GetStrPoint
 jr z,IS_LoopLink
 ld hl,$800D
 inc (hl)
 jr IS_LoopLink
GoLeft:
 ld hl,$800D
 ld a,(INP_STARTCOL)
 cp (hl)
 jr nc,IS_LoopLink
 dec (hl)
 jr IS_LoopLink
GoUp:
 ld a,(INP_STARTCOL)
 ld ($800D),a
 jr IS_LoopLink
GoDown:
 call &GetEnd
 ld a,(INP_STARTCOL)
 sub c
 dec a
 ld ($800D),a

IS_LoopLink:
 jp &IS_Loop

GoClear:
 call &GetEnd
 ld hl,(INP_STRADDR)
 ld (hl),a
 ld a,(INP_STARTCOL)
 ld b,a
 ld hl,$800D
 ld (hl),a
 ld a,$20		;ascii for ' '
GC_Loop:
 call TR_CHARPUT	;clear printed string
 inc (hl)
 inc c
 jr nz,GC_Loop
 ld (hl),b
 jr IS_LoopLink
GoDelete:
 call &GetStrPoint
 jr z,IS_LoopLink
 push hl
 call &GetEnd
 pop de
 sbc hl,de
 ld c,l
 dec c
 ld h,d
 ld l,e
 push hl
 inc hl
 ldir
 pop hl
 ld a,($800D)	;save column cursor is at
 call D_ZT_STR	;display the rest of the chars
 ld c,a
 ld a,$20
 call TR_CHARPUT
 ld a,c
 ld ($800D),a	;restore column
 jr IS_LoopLink


;Output:
;	C,B = keycode3
;	(HL) -> 2nd keycode3 byte ( =B)
;	A = keycode2
;	DE -> keycode3 table
GetKey3:
 call INP_WAITKEY	;waits for key press and returns A = keycode2

 res 3,(iy+$0C)	;cursor is no longer showing
 ld l,a
 ld h,0		;HL = A
 ld a,$46
 ld ($800B),a	;reset cursor-blink counter
 ld a,($800E)	;get char under cursor
 call TR_CHARPUT	;erase cursor
 ld a,(ROM_VERS)	;get ROM version
 ld de,$0669
 cp $2A
 jr nz,$+4
 ld e,$60		;DE -> keycode3 table (relative to ROM vers)

 ld a,l		;A = keycode2
 add hl,hl
 add hl,de
 ld c,(hl)		;C = keycode3
 inc hl
 ld b,(hl)		;B = $FF -> display char C (if C != 0)
 ret




;checks if the string's length is max or not
;Input:
;	(INP_STRADDR) = address to put string
;	(INP_MAXLENGTH) = max length of string
;Output:
;	A destroyed
;	B = 0
;	C = -(string length + 1)
;	HL -> byte after nul terminator
;	if NC, full string
;	if CF, string still has room
CheckLength:
 call &GetEnd
 ld a,(INP_MAXLENGTH)	;A = maximum length the inputted string can be
 add a,c		;add -C (or subtract C)
 ret

;gets end of string and length of string
;Input:
;	(INP_STRADDR) = address to put string
;Output:
;	A = B = 0
;	-C = length of inputted string + 1
;	HL -> byte after nul terminator
GetEnd:
 ld hl,(INP_STRADDR)
 xor a
 ld b,a
 ld c,a
 cpir
 inc b
 ret


;points HL to the char in string where cursor is at
;Input:
;	(INP_STRADDR) = address of string
;	(INP_STARTCOL) = starting column of cursor
;	($800D) = cursor column
;Output:
;	HL -> string position cursor is at
;	A = (HL)
;	B = 0
;	C = offset of cursor [(INP_STARTCOL) - ($800D)]
;	if NZ, A != 0
;	if ZF, A = 0
GetStrPoint:
 ld a,(INP_STARTCOL)
 ld c,a
 ld a,($800D)
 sub c
 ld c,a
 ld b,0
 ld hl,(INP_STRADDR)
 add hl,bc
 ld a,(hl)
 or a
 ret


InputInt:
 bit 2,(iy+$0C)
 ret z
 call &GetStrPoint
 jr nz,$+4
 ld a,$20
 ld ($800E),a
 ret


.end
